#include <LiquidCrystal.h>
//lcd(RS, E, D4, D5, D6, D7);
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

void setup() {
  Serial.begin(115200);
  lcd.begin(16,2);
  lcd.clear();
  lcd.print("* LCD  Smartie *");
  lcd.setCursor(0,1);
  lcd.print("* Franzis Nano *");
}

byte serial_getch() {
  int incoming;
  while (Serial.available()==0) {}
  incoming = Serial.read(); 
  return (byte) (incoming &0xff);
}

void loop() {
  byte rxbyte;
  byte temp;
  byte col;
  byte row;
  uint8_t location;
  uint8_t charMap[7];

  rxbyte = serial_getch();
  if (rxbyte == 254) //Matrix Orbital
  {
    switch (serial_getch())
    {
      case 71:  //Cursorposition setzen
        col = (serial_getch() - 1);
        row = (serial_getch());
        lcd.setCursor(col,row-1);
        break;
      case 72:  //Cursor Home
        lcd.setCursor(0,0);
        break;
      case 78:  //Benutzerdefinierte Zeichen
        location = serial_getch();
        for (temp = 0; temp < 8; temp++)
        {
          charMap[temp] = serial_getch();
        }
        lcd.createChar(location, charMap);
        break;
      case 88:  //Display löschen
        lcd.clear();
        lcd.setCursor(0,0);
        break;

      //Diese Kommandos werden ignoriert
      case 35: //read serial number
      case 36: //read version number
      case 55: //read module type
      case 59: //exit flow-control mode
      case 65: //auto transmit keypresses
      case 66: //backlight on
      case 96: //auto-repeat mode off (keypad)
      case 67: //auto line-wrap on
      case 68: //auto line-wrap off
      case 70: //backlight off
      case 74: //show underline cursor
      case 75: //underline cursor off
      case 76: //move cursor left
      case 77: //move cursor right
      case 81: //auto scroll on
      case 82: //auto scroll off
      case 83: //show blinking block cursor
      case 84: //block cursor off
      case 86: //GPO OFF
      case 87: //GPO ON
      case 104: //init horiz bar graph
      case 109: //init med size digits
      case 115: //init narrow vert bar graph
      case 118: //init wide vert bar graph
      case 152: //set and remember
      case 153: //set backlight brightness
        break;
      default:
        //alle anderen Kommandos werden ignoriert und das Parameter-Byte gelöscht
        temp = serial_getch();
        break;
    }
    return;
  } //ENDE DES KOMMANDO-HANDLERS

  switch (rxbyte)
  {
    //Zeichen, die im Zeichensatz enthalten sind
    case 0xC3: //ASCII "A" Tilde
      rxbyte = 0xAA;
      break;
    case 0xC4: //ASCII "A" Umlaut
      rxbyte = 0x8E;
      break;
    case 0xC5: //ASCII "A" Ring
      rxbyte = 0x8F;
      break;
    case 0xC6: //ASCII "AE" Ligatur 
      rxbyte = 0x92;
      break;
    case 0xC7: //ASCII "C" Cedilla
      rxbyte = 0x80;
      break;
    case 0xC9: //ASCII "E" Acute
      rxbyte = 0x90;
      break;
    case 0xD1: //ASCII "N" Tilde 
      rxbyte = 0x9C;
      break;
    case 0xD5: //ASCII "O" Tilde
      rxbyte = 0xAC;
      break;
    case 0xD6: //ASCII "O" Umlaut
      rxbyte = 0x99;
      break;
    case 0xD8: //ASCII "O" Strich
      rxbyte = 0xAE;
      break;
    case 0xDC: //ASCII "U" Umlaut
      rxbyte = 0x9A;
      break;
    case 0xDF: //ASCII "s" scharf
      rxbyte = 0xE0;
      break;
    case 0xE0: //ASCII "a" Gravis
      rxbyte = 0x85;
      break;
    case 0xE1: //ASCII "a" Acute
      rxbyte = 0xA0;
      break;
    case 0xE2: //ASCII "a" Zirkumflex
      rxbyte = 0x83;
      break;
    case 0xE3: //ASCII "a" Tilde
      rxbyte = 0xAB;
      break;
    case 0xE4: //ASCII "a" Umlaut
      rxbyte = 0x84;
      break;
    case 0xE5: //ASCII "a" Ring 
      rxbyte = 0x86;
      break;
    case 0xE6: //ASCII "ae" Ligatur 
      rxbyte = 0x91;
      break;
    case 0xE7: //ASCII "c" Cedilla
      rxbyte = 0x87;
      break;
    case 0xE8: //ASCII "e" Gravis
      rxbyte = 0x8A;
      break;
    case 0xE9: //ASCII "e" Acute
      rxbyte = 0x82;
      break;
    case 0xEA: //ASCII "e" Zirkumflex
      rxbyte = 0x88;
      break;
    case 0xEB: //ASCII "e" Trema
      rxbyte = 0x89;
      break;
    case 0xEC: //ASCII "i" Gravis
      rxbyte = 0x8D;
      break;
    case 0xED: //ASCII "i" Acute
      rxbyte = 0xA1;
      break;
    case 0xEE: //ASCII "i" Zirkumflex
      rxbyte = 0x8C;
      break;
    case 0xEF: //ASCII "i" Trema
      rxbyte = 0x8B;
      break;
    case 0xF1: //ASCII "n" Tilde
      rxbyte = 0x9B;
      break;
    case 0xF2: //ASCII "o" Gravis
      rxbyte = 0x95;
      break;
    case 0xF3: //ASCII "o" Acute
      rxbyte = 0xA2; 
      break;
    case 0xF4: //ASCII "o" Zirkumflex
      rxbyte = 0x93; 
      break;
    case 0xF5: //ASCII "o" Tilde
      rxbyte = 0xAD; 
      break;
    case 0xF6: //ASCII "o" Umlaut
      rxbyte = 0x94; 
      break;
    case 0xF8: //ASCII "o" Strich
      rxbyte = 0xAF;
      break;
    case 0xF9: //ASCII "u" Gravis
      rxbyte = 0x97;
      break;
    case 0xFA: //ASCII "u" Acute
      rxbyte = 0xA3;
      break;
    case 0xFB: //ASCII "u" Zirkumflex
      rxbyte = 0x96;
      break;
    case 0xFC: //ASCII "u" Umlaut
      rxbyte = 0x81;
      break;

    //Weitere Symbolumsetzungen
    case 0xA3: //Pfund Sterling 
      rxbyte = 0xA5;
      break;
    case 0xA7: //Paragraph 
      rxbyte = 0xD2;
      break;
    case 0xB2: //Hochgestellte 2
      rxbyte = 0x1E;
      break;
    case 0xB3: //Hochgestellte 3
      rxbyte = 0x1F;
      break;
    case 0xB5: //My
      rxbyte = 0xEA;
      break;
    case 0xC0: //"A" Varianten -> "A"
    case 0xC1:
    case 0xC2:
      rxbyte = 0x41;
      break;
    case 0xC8: //"E" Varianten -> "E"
    case 0xCA:
    case 0xCB:
      rxbyte = 0x45;
      break;
    case 0xCC: //"I" Varianten -> "I"
    case 0xCD:
    case 0xCE:
    case 0xCF:
      rxbyte = 0x49;
      break;
    case 0xD2: //"O" Varianten -> "O"
    case 0xD3:
    case 0xD4:
      rxbyte = 0x4F;
      break;
    case 0xD9: //"U" Varianten -> "U"
    case 0xDA:
    case 0xDB:
      rxbyte = 0x55;
      break;
    case 0xDD: //"Y" Acute -> "Y"
      rxbyte = 0x59;
      break;
    case 0xF7: //Divisionssymbol
      rxbyte = 0xB8;
      break;
    default:
      break;
  }

  lcd.write(rxbyte);
}
